#########################################################################
########################################################################
#### R codes to plot the figure 2, 3 and 4 
#####"Using the SAEM algorithm for mechanistic joint models 
##### characterizing the relationship between nonlinear PSA kinetics and survival in prostate cancer patients" 
############# Solne Desme, France Mentr, Christine Veyrat-Follet, Bernard Sbastien and Jrmie Guedj


# Load packages
library(survival)
library(deSolve)



# You have to estimate population parameters and individual parameters using Monolix before.

# set directory (where outputs of Monolix were saved)
setwd("~/your/path/to/Output_Monolix/")

# set directory (where figures will be saved)
way<-"~/your/path/to/Figures/"


# The estimated population parameters are (copy the precise values of the file "estimated.txt")
lambda=905.75160
beta1=0.00032
beta2=0.39485

sigma=0.31982

# Number of patients in the training dataset
N=400


#### Import individual parameters provided by Monolix
indiv_parameters <- read.delim("indiv_parameters.txt")

# RF_mode cannot be exactly equal to 1 because RF is logit-normally distributed : we have to replace by 0.99999
indiv_parameters$RF_mode <- ifelse(indiv_parameters$RF_mode==1, 0.99999, indiv_parameters$RF_mode)


#### Import data 
JM400 <- read.delim("JM400.txt")


### Only the PSA data
PSA400<-subset(JM400, YTYPE==1)
#### Merge the PSA data and the individual parameters
PSA<-merge(PSA400, indiv_parameters, by="ID")



#### Only survival data
survival400 <- subset(JM400, YTYPE==2 & TIME>0)





####################################################################
####################################################################
######## System of differential equations for the PSA kinetics
### Here we define a function which has a standard format for deSolve package. The keywords are:
# t : time
# y : function of dimension 3 composed of S (y1), R (y2) and PSA (y3)
# parameters : the vectors of the 6 longitudinal parameters
ED_PSA<-function(t, y, parameters) { 
  if  (t<=0){ # before treatment initiation (no epsilon)
    dy1<- parameters["alpha_S"]*(1-(y[1]+y[2])/parameters["Nmax"])*y[1]+0.0000001*(y[2]-y[1])-parameters["RE"]*parameters["RF"]*parameters["alpha_S"]*y[1]
    dy2<- parameters["RF"]*parameters["alpha_S"]*(1-(y[1]+y[2])/parameters["Nmax"])*y[2]+0.0000001*(y[1]-y[2])-parameters["RE"]*parameters["RF"]*parameters["alpha_S"]*y[2]
    dy3<- 20*(y[1]+y[2])-0.23*y[3]  }
  else { # from treatment initiation
    dy1<- parameters["alpha_S"]*(1-parameters["epsilon"])*(1-(y[1]+y[2])/parameters["Nmax"])*y[1]+0.0000001*(y[2]-y[1])-parameters["RE"]*parameters["RF"]*parameters["alpha_S"]*y[1]
    dy2<- parameters["RF"]*parameters["alpha_S"]*(1-(y[1]+y[2])/parameters["Nmax"])*y[2]+0.0000001*(y[1]-y[2])-parameters["RE"]*parameters["RF"]*parameters["alpha_S"]*y[2]
    dy3<- 20*(y[1]+y[2])-0.23*y[3]  
  }
  list(c(dy1, dy2, dy3)) 
}



##########################################################################
# Functions S(t), R(t) and PSA(t) 
Sens<-function(t,parameters){yini = c(0.23*parameters["PSAb"]/20,
                                      0.0000001/(parameters["RE"]*parameters["RF"]*parameters["alpha_S"]-parameters["RF"]*(0.0000001+parameters["RE"]*parameters["RF"]*parameters["alpha_S"]))*0.23*parameters["PSAb"]/20, 
                                      parameters["PSAb"])
                             max(lsoda(yini, t, ED_PSA, parameters )[,2], 1e-10)} # to avoid negative number of sensitive cells 
Resist<-function(t,parameters){yini = c(0.23*parameters["PSAb"]/20,
                                        0.0000001/(parameters["RE"]*parameters["RF"]*parameters["alpha_S"]-parameters["RF"]*(0.0000001+parameters["RE"]*parameters["RF"]*parameters["alpha_S"]))*0.23*parameters["PSAb"]/20, 
                                        parameters["PSAb"])
                               lsoda(yini, t, ED_PSA, parameters )[,3]}
PSA<-function(t,parameters){yini = c(0.23*parameters["PSAb"]/20,
                                     0.0000001/(parameters["RE"]*parameters["RF"]*parameters["alpha_S"]-parameters["RF"]*(0.0000001+parameters["RE"]*parameters["RF"]*parameters["alpha_S"]))*0.23*parameters["PSAb"]/20, 
                                     parameters["PSAb"])
                            lsoda(yini, t, ED_PSA, parameters )[,4]}



# Hazard function depends on S(t) and R(t)
haz<-function(t, parameters){
  ifelse(t<0, 0 , 1/lambda*exp(beta1*log(Sens(t, parameters))+beta2*log(Resist(t, parameters))) )
}










####################################################################
####################################################################
#####################################################################
########## Individual PSA profiles and survival
##### (Figure 2)

### In 9 selected patients  

sample9<-c(2026, 2031, 2040, 2108, 2180, 2330,2524, 2549, 2598)


# Computation of their PSA kinetics every day
pred<-NULL
for (i in sample9){
  parameters=c(alpha_S=indiv_parameters$alpha_S_mode[indiv_parameters$ID==i],
               RF=indiv_parameters$RF_mode[indiv_parameters$ID==i],
               RE=indiv_parameters$RE_mode[indiv_parameters$ID==i],              
               epsilon=indiv_parameters$epsilon_mode[indiv_parameters$ID==i],               
               PSAb=indiv_parameters$PSAb_mode[indiv_parameters$ID==i],
               Nmax=indiv_parameters$Nmax_mode[indiv_parameters$ID==i]
  ) # Individual parameters for PSA provided by Monolix
  
  yini = c(0.23*parameters["PSAb"]/20,
           0.0000001/(parameters["RE"]*parameters["RF"]*parameters["alpha_S"]-parameters["RF"]*(0.0000001+parameters["RE"]*parameters["RF"]*parameters["alpha_S"]))*0.23*parameters["PSAb"]/20, 
           parameters["PSAb"]) # Initial values for the system of ODEs
  
  times=seq(from=min(PSA400$TIME[PSA400$ID==i]), to=survival400$TIME[survival400$ID==i], by=1) # times at which compute the values of y (here every 1 day from the first PSA measurement to the time-to-event)
  
  out <- data.frame(ode(y = yini, times = times, func = ED_PSA, parms = parameters)) # Use of the ode function of the deSolve package to compute the integral
  out<-cbind(i,out)
  pred<-rbind(pred,out)
  
}

colnames(pred)<-c("ID","time","S","R","PSA")

# We use the transformation: log(PSA+1)
pred$logPSA<-log(pred$PSA+1)





## Computation of the individual hazard function
pred$haz<-0
for (i in unique(pred$ID)){  
  pred[pred$ID==i,]$haz<-haz(t=pred$time[pred$ID==i], parameters=c(alpha_S=indiv_parameters$alpha_S_mode[indiv_parameters$ID==i],
                                                                   RF=indiv_parameters$RF_mode[indiv_parameters$ID==i],
                                                                   RE=indiv_parameters$RE_mode[indiv_parameters$ID==i],
                                                                   epsilon=indiv_parameters$epsilon_mode[indiv_parameters$ID==i], 
                                                                   PSAb=indiv_parameters$PSAb_mode[indiv_parameters$ID==i],
                                                                   Nmax=indiv_parameters$Nmax_mode[indiv_parameters$ID==i]))
}


## Cumulative hazard (from time 0 !)
pred2<-subset(pred, time>=0)

pred2$hazard_cum<-0
for (i in unique(pred2$ID)){
  pred2[pred2$ID==i,]$hazard_cum<-cumsum(pred2[pred2$ID==i,]$haz) 
}

#survival
pred2$survival<-exp(-pred2$hazard_cum)



##############################################################################
#Figure 2
png(filename = paste(way,"//Indiv_fits.png", sep=""),
    width = 3500, height = 2500, units = "px", res=300)
par(mfrow=c(3,3))
for (i in sample9){
  predPSA<-subset(pred, ID==i) # predicted PSA in patient i
  obsPSA<-subset(PSA400, ID==i) # observed PSA in patient i
  predSURV<-subset(pred2, ID==i & time>0) # predicted survival in patient i
  obsSURV<-subset(survival400, ID==i) # observed time-to-event in patient i
  
  plot.new()
  par(mar=c(4,4,2,4))
  plot.window(xlim=c(-500,1150), ylim=c(0,8))
  
  #Period of treatment 
  rect(0, 0, unique(obsPSA$TIME_ENDtr), 8, col = "lightgrey", border = "transparent")
  
  points(x=obsPSA$TIME, y=obsPSA$Y, col="dodgerblue3", pch=3, cex=2)
  lines(x=predPSA$time, y=log(predPSA$PSA+1), col="seagreen4", lwd=2)
  
  box(col = "black")
  axis(1)
  axis(2, at=c(0,log(11), log(101), log(1001), log(10001)), labels=c(0,10,10^2,10^3,10^4))
  plot.window(xlim=c(-500,1150), ylim=c(0,0.005))
  lines(x=predSURV$time, y=predSURV$haz, col="red3", lwd=2, lty="dashed")
  abline(v = obsSURV$TIME, lty =ifelse(obsSURV$Y==0, 3, 1), lwd=2, col="dimgrey")
  
  axis(4, at=c(0,0.0025,0.005))
  if (i==2330) {mtext(expression(paste("Hazard (",day^{-1},")")),side=4,line=3)}
  if (i==2108) {mtext(expression(paste("PSA (",ng.mL^{-1},")")), side=2, line=2)}
  if (i==2549) {mtext("Time since treatment initiation (days)", side=1, line=2.5)}
  title(paste("Patient",i), line=0.5)
}

dev.off()














#########################################################################
########################################################################
####################################
########### Residuals
######### (Figure 3)



#####################
#### For PSA
## IWRES

pred2<-NULL
for (i in unique(indiv_parameters$ID)){
  parameters=c(alpha_S=indiv_parameters$alpha_S_mode[indiv_parameters$ID==i],
               RF=indiv_parameters$RF_mode[indiv_parameters$ID==i],
               RE=indiv_parameters$RE_mode[indiv_parameters$ID==i],               
               epsilon=indiv_parameters$epsilon_mode[indiv_parameters$ID==i],               
               PSAb=indiv_parameters$PSAb_mode[indiv_parameters$ID==i],
               Nmax=indiv_parameters$Nmax_mode[indiv_parameters$ID==i]
  )
  yini = c(0.23*parameters["PSAb"]/20,
           0.0000001/(parameters["RE"]*parameters["RF"]*parameters["alpha_S"]-parameters["RF"]*(0.0000001+parameters["RE"]*parameters["RF"]*parameters["alpha_S"]))*0.23*parameters["PSAb"]/20, 
           parameters["PSAb"])
  times=subset(PSA400, ID==i)$TIME # times at which compute the values of y (here every time of observed PSA measurement)
  
  out <- data.frame(ode(y = yini, times = times, func = ED_PSA, parms = parameters))
  out<-cbind(i,out)
  pred2<-rbind(pred2,out)
  
}

colnames(pred2)<-c("ID","TIME","S","R","PSA_pred")

pred2$logPSA_pred<-log(pred2$PSA_pred+1)

pred2<-merge(pred2,PSA400, by=c("ID","TIME"))


## IWRES is the difference between the observed log(PSA+1) and the predicted log(PSA+1) divided by sigma
pred2$iwres<-(pred2$Y-pred2$logPSA_pred)/sigma








######################################################
#####################################################
##################### For survival
### Cox-Snell residuals

pred<-NULL
for (i in unique(indiv_parameters$ID)){
  parameters=c(alpha_S=indiv_parameters$alpha_S_mode[indiv_parameters$ID==i],
               RF=indiv_parameters$RF_mode[indiv_parameters$ID==i],
               RE=indiv_parameters$RE_mode[indiv_parameters$ID==i],               
               epsilon=indiv_parameters$epsilon_mode[indiv_parameters$ID==i],               
               PSAb=indiv_parameters$PSAb_mode[indiv_parameters$ID==i],
               Nmax=indiv_parameters$Nmax_mode[indiv_parameters$ID==i]
  )
  yini = c(0.23*parameters["PSAb"]/20,
           0.0000001/(parameters["RE"]*parameters["RF"]*parameters["alpha_S"]-parameters["RF"]*(0.0000001+parameters["RE"]*parameters["RF"]*parameters["alpha_S"]))*0.23*parameters["PSAb"]/20, 
           parameters["PSAb"])
  
  times=c(min(PSA400$TIME[PSA400$ID==i]) , seq(from=0, to=survival400$TIME[survival400$ID==i], by=1) ) # times at which compute the values of y (here every 1 day from the first PSA measurement to the time-to-event)
  
  out <- data.frame(ode(y = yini, times = times, func = ED_PSA, parms = parameters))
  out<-cbind(i,out)
  pred<-rbind(pred,out)
  
}

colnames(pred)<-c("ID","time","S","R","PSA")


pred<-merge(pred, indiv_parameters, by="ID")


#### Hazard computation
pred$haz<-0

for (i in unique(pred$ID)){  
  pred[pred$ID==i,]$haz<-haz(t=pred$time[pred$ID==i], parameters=c(alpha_S=indiv_parameters$alpha_S_mode[indiv_parameters$ID==i],
                                                                   RF=indiv_parameters$RF_mode[indiv_parameters$ID==i],
                                                                   RE=indiv_parameters$RE_mode[indiv_parameters$ID==i],                                                                  
                                                                   epsilon=indiv_parameters$epsilon_mode[indiv_parameters$ID==i],                                             
                                                                   PSAb=indiv_parameters$PSAb_mode[indiv_parameters$ID==i],
                                                                   Nmax=indiv_parameters$Nmax_mode[indiv_parameters$ID==i]))
}


#### Cumulative hazard function

pred<-subset(pred, time>=0)

pred$hazard_cum<-0
for (i in unique(pred$ID)){
  pred[pred$ID==i,]$hazard_cum<-cumsum(pred[pred$ID==i,]$haz) 
}


# Cox-Snell residuals are the cumulative hazard function at the time of death (or censure)
pred<-pred[order(pred$time, decreasing = TRUE),]
pred<-pred[order(pred$ID),]
pred_CS<-pred[!duplicated(pred$ID),]

pred_CS<-merge(survival400, pred_CS, by="ID")




#########################################################
#########################################################
########## Martingale residuals

pred_CS$res_martingale<-pred_CS$Y-pred_CS$hazard_cum


# Martingale residuals will be plotted versus predicted log(S) and log(R)
pred_CS$logS<-log(pred_CS$S)
pred_CS$logR<-log(pred_CS$R)








############################################
### ### All residuals on one sheet


png(filename = paste(way,"//residuals.png", sep=""),
    width = 3000, height = 5000, units = "px", res=440)

par(mfrow=c(1,1))
par(mar=c(4,4,2,2))

split1<-split.screen(c(3, 1))
split2<-split.screen(c(1, 2), screen = 1)  ## to split the screen
split3<-split.screen(c(1, 2), screen = 3)

screen(split2[1])
# IWRES vs time
plot(pred2$TIME, pred2$iwres, type="p", pch=20, col="dodgerblue4",
     xaxt = "n", yaxt = "n", xlab="Time since the treatment initiation (days)", ylab="IWRES")
abline(h=0)
axis(1)
axis(2)

screen(split2[2])
# IWRES vs predicted PSA
plot(pred2$logPSA_pred, pred2$iwres, type="p", pch=20, col="dodgerblue4",
     xaxt = "n", yaxt = "n", xlab=expression(paste("Predicted PSA (",ng.mL^{-1},")")), ylab="IWRES")
abline(h=0)
axis(1, at=c(log(0.000001),log(0.00001),log(0.0001),log(0.001),log(0.01),log(0.1),0,log(10), log(100), log(1000), log(10000)), labels=c(10^{-6},10^{-5},10^{-4},10^{-3},10^{-2},10^{-1},0,10,10^2,10^3,10^4))
axis(2)

screen(split1[2])
## Cox-Snell residuals
surv<-survfit(Surv(hazard_cum,Y)~1,data=pred_CS)
plot(surv, xlab = "Cox-Snell residuals", ylab = "Survival probability", mark.time=F, lwd=2)
# compared to exp(-r)
r=seq(from=0, to=max(pred_CS$hazard_cum), by=0.01)
lines(r, exp(-r), col="grey", lwd=3)


screen(split3[1])
## Martingale residuals
# vs predicted log(S)
plot(pred_CS$logS, pred_CS$res_martingale, type="p", pch=18, col="dodgerblue4",
     xaxt = "n", yaxt = "n", xlab="Predicted count of sensitive cells", ylab="Martingale residuals")
abline(h=0)
axis(1, at=c(log(0.000001),log(0.00001),log(0.0001),log(0.001),log(0.01),log(0.1),0,log(10), log(100), log(1000), log(10000)), labels=c(10^{-6},10^{-5},10^{-4},10^{-3},10^{-2},10^{-1},0,10,10^2,10^3,10^4))
axis(2, at=c(-6:1))

screen(split3[2])
# vs predicted log(R)
plot(pred_CS$logR, pred_CS$res_martingale, type="p", pch=18, col="dodgerblue4",
     xaxt = "n", yaxt = "n", xlab="Predicted count of resistant cells", ylab="Martingale residuals")
abline(h=0)
axis(1, at=c(log(0.000001),log(0.00001),log(0.0001),log(0.001),log(0.01),log(0.1),0,log(10), log(100), log(1000), log(10000)), labels=c(10^{-6},10^{-5},10^{-4},10^{-3},10^{-2},10^{-1},0,10,10^2,10^3,10^4))
axis(2, at=c(-6:1))


close.screen(all = TRUE)

dev.off()



























########################################################
#######################################################
#######################################################
###### Mean survival curve
##### (Figure 4)

pred<-NULL
for (i in unique(indiv_parameters$ID)){
  parameters=c(alpha_S=indiv_parameters$alpha_S_mode[indiv_parameters$ID==i],
               RF=indiv_parameters$RF_mode[indiv_parameters$ID==i],
               RE=indiv_parameters$RE_mode[indiv_parameters$ID==i],               
               epsilon=indiv_parameters$epsilon_mode[indiv_parameters$ID==i],              
               PSAb=indiv_parameters$PSAb_mode[indiv_parameters$ID==i],
               Nmax=indiv_parameters$Nmax_mode[indiv_parameters$ID==i]
  )
  yini = c(0.23*parameters["PSAb"]/20,
           0.0000001/(parameters["RE"]*parameters["RF"]*parameters["alpha_S"]-parameters["RF"]*(0.0000001+parameters["RE"]*parameters["RF"]*parameters["alpha_S"]))*0.23*parameters["PSAb"]/20, 
           parameters["PSAb"])
  times=c(min(PSA400$TIME[PSA400$ID==i]) , seq(from=0, to=1570, by=10) )  # times at which compute the values of y (here every 10 day from the first PSA measurement to 1570 (the greatest observed time-to-event))
                                                                           ###### by 10 days to decrease runtimes (can be set to 1 without difference in the figure)
  
  out <- data.frame(ode(y = yini, times = times, func = ED_PSA, parms = parameters))
  out<-cbind(i,out)
  pred<-rbind(pred,out)
  
}

colnames(pred)<-c("ID","time","S","R","PSA")

pred<-merge(pred, indiv_parameters, by="ID")



pred$haz<-0
for (i in unique(pred$ID)){  
  pred[pred$ID==i,]$haz<-haz(t=pred$time[pred$ID==i], parameters=c(alpha_S=indiv_parameters$alpha_S_mode[indiv_parameters$ID==i],
                                                                   RF=indiv_parameters$RF_mode[indiv_parameters$ID==i],
                                                                   RE=indiv_parameters$RE_mode[indiv_parameters$ID==i],                                                                  
                                                                   epsilon=indiv_parameters$epsilon_mode[indiv_parameters$ID==i],                                              
                                                                   PSAb=indiv_parameters$PSAb_mode[indiv_parameters$ID==i],
                                                                   Nmax=indiv_parameters$Nmax_mode[indiv_parameters$ID==i]))
}



pred<-subset(pred, time>=0)

#### Cumulative hazard

pred$hazard_cum<-0
for (i in unique(pred$ID)){
  pred[pred$ID==i,]$hazard_cum<-cumsum(pred[pred$ID==i,]$haz) *10 ### because by=10 days...
}




#survival
pred$survival<-exp(-pred$hazard_cum)



### Mean survival curve
surv_marg<-function(t){
  1/N*sum(subset(pred, time==t)$survival)
}  

times=seq(from=0, to=max(survival400$TIME), by=10) 

survival_marg<-rep(0,length(times))
for (i in 1:length(times)){
  survival_marg[i]<-surv_marg(times[i])
}






### Figure 4 for the training dataset
png(filename = paste(way,"//KM_survival.png", sep=""),
    width = 2000, height = 1600, units = "px", res=400)
par(mfrow=c(1,1), mar=c(4,4,2,2))
surv<-survfit(Surv(TIME,Y)~1,data=survival400)
plot(surv, xlab = "Time since treatment initiation (days)", ylab = "Survival", mark.time=F, col="grey",lwd=2) # plot the KM curve
lines(times, survival_marg, col="red3", lwd=3) # plot the mean survival curve

dev.off()
















